@@ -87,7 +87,7 @@ file.
 =head1 DESCRIPTION
 
 StreamFinder accepts a valid radio station URL on supported websites and
-returns the actual stream URL, title, and cover art icon for that station.  
+returns the actual stream URL(s), title, and cover art icon for that station.  
 The purpose is that one needs one of these URLs 
 in order to have the option to stream the station/video in one's own choice of 
 audio player software rather than using their web browser and accepting any / 
@@ -96,7 +96,7 @@ that can come with that method of playing.  The author uses his own custom
 all-purpose media player called "fauxdacious" (his custom hacked version of 
 the open-source "audacious" audio player).  "fauxdacious" incorporates this 
 module to decode and play streams.  The currently-supported websites are:
-radionomy.com, iheartradio.com, reciva.com, facebook, banned.video, 
+radionomy.com, iheartradio.com, reciva.com, radio.net, facebook, banned.video, 
 and youtube / vimeo / brighteon, et. al.  We are again supporting tunein.com, 
 but it may not work for all stations!
 
@@ -306,7 +306,7 @@ use strict;
 use warnings;
 use vars qw(@ISA @EXPORT $VERSION);
 
-our $VERSION = '1.10';
+our $VERSION = '1.11';
 our $DEBUG = 1;
 
 require Exporter;
