@@ -94,33 +94,35 @@ One or more stream URLs can be returned for each video.
 
 =over 4
 
-=item B<new>(I<url> [, I<streamtype> | "debug" [ => 0|1|2 ] ... ])
+=item B<new>(I<url> [, I<-keep> => "type1[,type2...]"|I<[> type1[,type2...] I<]> | , I<-debug> [ => 0|1|2 ] ])
 
 Accepts an Banned.Video URL and creates and returns a new station object, or 
 I<undef> if the URL is not a valid Banned.Video station or no streams are found.
 
-The optional I<streamtype> can be one of:  any, secure, secure_pls, pls, 
-secure_hls, hls, secure_shortcast, shortcast, secure_rtmp, rtmp, etc.  More 
-than one value can be specified to control order of search.  A I<streamtype> 
-can be preceeded by an exclamantion point ("!") to reject that type of stream.
-If "any" appears in the list, it should be the last specifier without a "!" 
-preceeding it, and itself should not be preceeded with a "!" (inverter)!  
-
-For example, the list:  'secure_shoutcast', 'secure', 'any', '!rtmp' 
-would try to find a "secure_shoutcast" (https) shortcast stream, if none found, 
-would then look for any secure (https) stream, failing that, would look for 
-any valid stream.  All the while skipping any that are "rtmp" 
-streams.
+The optional I<keep> can be either a comma-separated string or an array reference 
+([...]) of stream types to keep (include) and returned in order specified 
+(type1, type2...).  Each "type" can be one of:  extension (ie. m4a, mp4, etc.), 
+"direct", "stream", or ("any" or "all").  The default list is:  'm4a,direct,stream',
+meaning that all m4a streams followed by all "direct" streams ("directUrl" in page, 
+followed by all remaining (non-direct "streamUrl") streams.  More 
+than one value can be specified to control order of search.   
 
 =item $station->B<get>()
 
 Returns an array of strings representing all stream urls found.
 
-=item $station->B<getURL>()
+=item $station->B<getURL>([I<options>])
 
 Similar to B<get>() except it only returns a single stream representing 
 the first valid stream found.  
 
+Current options are:  I<-random> and I<-noplaylists>.  By default, the 
+first ("best"?) stream is returned.  If I<-random> is specified, then 
+a random one is selected from the list of streams found.  
+If I<-noplaylists> is specified, and the stream to be returned is a 
+"playlist" (.pls or .m3u8 extension), it is first fetched and the first entry 
+in the playlist is returned.  This is needed by Fauxdacious Mediaplayer.
+
 =item $station->B<count>()
 
 Returns the number of streams found for the station.
@@ -157,6 +159,44 @@ Returns the stream's type ("BannedVideo").
 
 =back
 
+=head1 CONFIGURATION FILES
+
+=over 4
+
+=item ~/.config/StreamFinder/BannedVideo/config
+
+Optional text file for specifying various configuration options 
+for a specific site (submodule).  Each option is specified on a 
+separate line in the format below:
+
+'option' => 'value' [,]
+
+and the options are loaded into a hash used only by the specific 
+(submodule) specified.  Valid options include 
+I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.  
+Blank lines and lines starting with a "#" sign are ignored.
+
+Options specified here override any specified in I<~/.config/StreamFinder/config>.
+
+Among options valid for BannedVideo streams is the I<-keep> option 
+described in the B<new()> function.
+
+=item ~/.config/StreamFinder/config
+
+Optional text file for specifying various configuration options.  
+Each option is specified on a separate line in the format below:
+
+'option' => 'value' [,]
+
+and the options are loaded into a hash used by all sites 
+(submodules) that support them.  Valid options include 
+I<-debug> => [0|1|2], and most of the L<LWP::UserAgent> options.
+
+=back
+
+NOTE:  Options specified in the options parameter list will override 
+those corresponding options specified in these files.
+
 =head1 KEYWORDS
 
 BannedVideo
@@ -252,7 +292,9 @@ use warnings;
 use LWP::UserAgent ();
 use vars qw(@ISA @EXPORT);
 
-our $DEBUG = 0;
+my $DEBUG = 0;
+my %uops = ();
+my @userAgentOps = ();
 
 require Exporter;
 
@@ -263,25 +305,55 @@ sub new
 {
 	my $class = shift;
 	my $url = shift;
-	my (@okStreams, @skipStreams);
+
+	my $self = {};
+	return undef  unless ($url);
+
+	my @okStreams;
+
+	foreach my $p ("$ENV{HOME}/.config/StreamFinder/config", "$ENV{HOME}/.config/StreamFinder/Youtube/config") {
+		if (open IN, $p) {
+			my ($atr, $val);
+			while (<IN>) {
+				chomp;
+				next  if (/^\s*\#/o);
+				($atr, $val) = split(/\s*\=\>\s*/o, $_, 2);
+				eval "\$uops{$atr} = $val";
+			}
+			close IN;
+		}
+	}
+	foreach my $i (qw(agent from conn_cache default_headers local_address ssl_opts max_size
+			max_redirect parse_head protocols_allowed protocols_forbidden requests_redirectable
+			proxy no_proxy)) {
+		push @userAgentOps, $i, $uops{$i}  if (defined $uops{$i});
+	}
+	push (@userAgentOps, 'agent', 'Mozilla/5.0 (X11; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0')
+			unless (defined $uops{'agent'});
+	$uops{'timeout'} = 10  unless (defined $uops{'timeout'});
+	$DEBUG = $uops{'debug'}  if (defined $uops{'debug'});
+
 	while (@_) {
-		if ($_[0] =~ /^\!/o) {
-			(my $i = shift) =~ s/\!//o;
-			push @skipStreams, $i;
-		} elsif ($_[0] =~ /^\-?debug$/o) {
+		if ($_[0] =~ /^\-?debug$/o) {
 			shift;
 			$DEBUG = (defined($_[0]) && $_[0] =~/^[0-9]$/) ? shift : 1;
 print STDERR "-???- DEBUG=$DEBUG=\n";
+		} elsif ($_[0] =~ /^\-?keep$/o) {
+			shift;
+			if (defined $_[0]) {
+				my $keeporder = shift;
+				@okStreams = (ref($keeporder) =~ /ARRAY/) ? @{$keeporder} : split(/\,\s*/, $keeporder);
+			}
 		} else {
-			push @okStreams, shift;
+			shift;
 		}
 	}	
-	@okStreams = ('any')  unless (defined $okStreams[0]);  # one of:  {secure_pls | pls | stw}
-
-	my $self = {};
+	if (!defined($okStreams[0]) && defined($uops{'keep'})) {
+		@okStreams = (ref($uops{'keep'}) =~ /ARRAY/) ? @{$uops{'keep'}} : split(/\,\s*/, $uops{'keep'});
+	}
+	@okStreams = (qw(m4a direct stream))  unless (defined $okStreams[0]);  # one of:  {m4a, <ext>, direct, stream, any, all}
 
 	print STDERR "-0(BannedVideo): URL=$url=\n"  if ($DEBUG);
-	return undef  unless ($url);
 
 	(my $url2fetch = $url);
 	$url2fetch = 'https://banned.video/watch?id=' . $url  unless ($url =~ /^http/);
@@ -289,8 +361,8 @@ print STDERR "-???- DEBUG=$DEBUG=\n";
 	$self->{'id'} = $1  if ($url2fetch =~ m#id\=([^\/\?\&]+)$#);
 	my $html = '';
 	print STDERR "-0(BannedVideo): ID=".$self->{'id'}."= URL=$url2fetch=\n"  if ($DEBUG);
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $response = $ua->get($url2fetch);
@@ -298,8 +370,11 @@ print STDERR "-???- DEBUG=$DEBUG=\n";
 		$html = $response->decoded_content;
 	} else {
 		print STDERR $response->status_line  if ($DEBUG);
-		print STDERR "\n..trying wget...\n"  if ($DEBUG);
-		$html = `wget -t 2 -T 20 -O- -o /dev/null \"$url2fetch\" 2>/dev/null `;
+		my $no_wget = system('wget','-V');
+		unless ($no_wget) {
+			print STDERR "\n..trying wget...\n"  if ($DEBUG);
+			$html = `wget -t 2 -T 20 -O- -o /dev/null \"$url2fetch\" 2>/dev/null `;
+		}
 	}
 	print STDERR "-1: html=$html=\n"  if ($DEBUG > 1);
 	return undef  unless ($html);  #STEP 1 FAILED, INVALID STATION URL, PUNT!
@@ -307,13 +382,18 @@ print STDERR "-???- DEBUG=$DEBUG=\n";
 	my $stindex = 0;
 	my @streams = ();
 	foreach my $streamtype (@okStreams) {
-		unless ($streamtype =~ /^m3u8$/i) {
-			$streams[$stindex] = ($html =~ /\"directUrl\"\s*\:\s*\"([^\"]+)\"/i) ? $1 : '';
-			$stindex++;
+		while ($html =~ s/\"(?:direct|stream)Url\"\s*\:\s*\"([^\"]+?\.${streamtype}\b[^\"]*)\"//i) {
+			$streams[$stindex++] = $1;
 		}
-		unless ($streamtype =~ /^(?:mp4|direct)$/i) {
-			$streams[$stindex] = ($html =~ /\"streamUrl\"\s*\:\s*\"([^\"]+)\"/i) ? $1 : '';
-			$stindex++;
+		if ($streamtype =~ /^(direct|stream)$/i) {
+			my $one = $1;
+			while ($html =~ s/\"${one}Url\"\s*\:\s*\"([^\"]+)\"//i) {
+				$streams[$stindex++] = $1;
+			}
+		} elsif ($streamtype =~ /^a(?:ny|ll)$/i) {
+			while ($html =~ s/\"(?:direct|stream)Url\"\s*\:\s*\"([^\"]+)\"//i) {
+				$streams[$stindex++] = $1;
+			}
 		}
 	}
 	print STDERR "-2: 1=$streams[0]= 2=$streams[1]\n"  if ($DEBUG);
@@ -344,8 +424,58 @@ sub get
 
 sub getURL   #LIKE GET, BUT ONLY RETURN THE SINGLE ONE W/BEST BANDWIDTH AND RELIABILITY:
 {
-	my $self = $_[0];
-	return ${$self->{'streams'}}[0];
+	my $self = shift;
+	my $arglist = (defined $_[0]) ? join('|',@_) : '';
+	my $idx = ($arglist =~ /\b\-?random\b/) ? int rand scalar @{$self->{'streams'}} : 0;
+	if ($arglist =~ /\b\-?noplaylists\b/ && ${$self->{'streams'}}[$idx] =~ /\.(pls|m3u8)$/i) {
+		my $plType = $1;
+		my $firstStream = ${$self->{'streams'}}[$idx];
+		print STDERR "-getURL($idx): NOPLAYLISTS and (".${$self->{'streams'}}[$idx].")\n"  if ($DEBUG);
+		my $ua = LWP::UserAgent->new(@userAgentOps);		
+		$ua->timeout($uops{'timeout'});
+		$ua->cookie_jar({});
+		$ua->env_proxy;
+		my $html = '';
+		my $response = $ua->get($firstStream);
+		if ($response->is_success) {
+			$html = $response->decoded_content;
+		} else {
+			print STDERR $response->status_line  if ($DEBUG);
+			my $no_wget = system('wget','-V');
+			unless ($no_wget) {
+				print STDERR "\n..trying wget...\n"  if ($DEBUG);
+				$html = `wget -t 2 -T 20 -O- -o /dev/null \"$firstStream\" 2>/dev/null `;
+			}
+		}
+		my @lines = split(/\r?\n/, $html);
+		$firstStream = '';
+		if ($plType =~ /pls/) {  #PLS:
+			my $firstTitle = '';
+			foreach my $line (@lines) {
+				if ($line =~ m#^\s*File\d+\=(.+)$#) {
+					$firstStream ||= $1;
+				} elsif ($line =~ m#^\s*Title\d+\=(.+)$#) {
+					$firstTitle ||= $1;
+				}
+			}
+			$self->{'title'} ||= $firstTitle;
+			print STDERR "-getURL(PLS): first=$firstStream= title=$firstTitle=\n"  if ($DEBUG);
+		} else {  #m3u8:
+			(my $urlpath = ${$self->{'streams'}}[$idx]) =~ s#[^\/]+$##;
+			foreach my $line (@lines) {
+				if ($line =~ m#^\s*([^\#].+)$#) {
+					my $urlpart = $1;
+					$urlpart =~ s#^\s+##;
+					$urlpart =~ s#^\/##;
+					$firstStream = ($urlpart =~ m#https?\:#) ? $urlpart : ($urlpath . '/' . $urlpart);
+					last;
+				}
+			}
+			print STDERR "-getURL(m3u8): first=$firstStream=\n"  if ($DEBUG);
+		}
+		return $firstStream || ${$self->{'streams'}}[$idx];
+	}
+	return ${$self->{'streams'}}[$idx];
 }
 
 sub count
@@ -383,8 +513,8 @@ sub getIconData
 	my $self = shift;
 	return ()  unless ($self->{'iconurl'});
 
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $art_image = '';
@@ -393,9 +523,12 @@ sub getIconData
 		$art_image = $response->decoded_content;
 	} else {
 		print STDERR $response->status_line  if ($DEBUG);
-		print STDERR "\n..trying wget...\n"  if ($DEBUG);
-		my $iconUrl = $self->{'iconurl'};
-		$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$iconUrl\" 2>/dev/null `;
+		my $no_wget = system('wget','-V');
+		unless ($no_wget) {
+			print STDERR "\n..trying wget...\n"  if ($DEBUG);
+			my $iconUrl = $self->{'iconurl'};
+			$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$iconUrl\" 2>/dev/null `;
+		}
 	}
 	return ()  unless ($art_image);
 
@@ -415,8 +548,8 @@ sub getImageData
 {
 	my $self = shift;
 	return ()  unless ($self->{'imageurl'});
-	my $ua = LWP::UserAgent->new;		
-	$ua->timeout(10);
+	my $ua = LWP::UserAgent->new(@userAgentOps);		
+	$ua->timeout($uops{'timeout'});
 	$ua->cookie_jar({});
 	$ua->env_proxy;
 	my $art_image = '';
@@ -425,6 +558,12 @@ sub getImageData
 		$art_image = $response->decoded_content;
 	} else {
 		print STDERR $response->status_line  if ($DEBUG);
+		my $no_wget = system('wget','-V');
+		unless ($no_wget) {
+			print STDERR "\n..trying wget...\n"  if ($DEBUG);
+			my $iconUrl = $self->{'iconurl'};
+			$art_image = `wget -t 2 -T 20 -O- -o /dev/null \"$iconUrl\" 2>/dev/null `;
+		}
 	}
 	return ()  unless ($art_image);
 	my $image_ext = $self->{'imageurl'};
