@@ -471,7 +471,7 @@ sub getURL   #LIKE GET, BUT ONLY RETURN THE SINGLE ONE W/BEST BANDWIDTH AND RELI
 					my $urlpart = $1;
 					$urlpart =~ s#^\s+##;
 					$urlpart =~ s#^\/##;
-					$firstStream = ($urlpart =~ m#https?\:#) ? $urlpart : ($urlpath . '/' . $urlpart);
+					$firstStream = ($urlpart =~ m#https?\:#) ? $urlpart : ($urlpath .  $urlpart);
 					last;
 				}
 			}
